@@ -1,11 +1,11 @@
-from matrx.agents.agent_brain import AgentBrain
+from matrx.agents import AgentBrain
 from matrx.agents.agent_utils.navigator import Navigator
 from matrx.agents.agent_utils.state_tracker import StateTracker
 
 
 class PatrollingAgentBrain(AgentBrain):
 
-    def __init__(self, waypoints, move_speed=1):
+    def __init__(self, waypoints, move_speed=0):
         super().__init__()
         self.state_tracker = None
         self.navigator = None
