@@ -1,11 +1,14 @@
+import os
+
 from matrx.agents.agent_types.patrolling_agent import PatrollingAgentBrain
 from matrx.logger.log_agent_actions import LogActions
 from matrx.world_builder import WorldBuilder
 
 
 def create_builder():
-    factory = WorldBuilder(random_seed=1, shape=[15, 15], tick_duration=0.2, verbose=True, run_matrx_api=True,
-                           run_matrx_visualizer=True)
+    tick_dur = 0.2
+    factory = WorldBuilder(random_seed=1, shape=[15, 15], tick_duration=tick_dur, verbose=True, run_matrx_api=True,
+                           run_matrx_visualizer=True, simulation_goal=int(300/tick_dur))
 
     factory.add_logger(logger_class=LogActions, save_path="log_data/")
 
@@ -38,3 +41,18 @@ def create_builder():
         factory.add_agent(start, navigating_agent, name="navigate " + str(x), visualize_shape=0, has_menu=True)
 
     return factory
+
+
+def run_simple_case(nr_of_worlds=1):
+    builder = create_builder()
+
+    # startup world-overarching MATRX scripts, such as the api and/or visualizer if requested
+    media_folder = os.path.dirname(os.path.realpath(__file__))  # set our path for media files to our current folder
+    builder.startup(media_folder=media_folder)
+
+    # run each world
+    for world in builder.worlds(nr_of_worlds=nr_of_worlds):
+        world.run(builder.api_info)
+
+    # stop MATRX scripts such as the api and visualizer (if used)
+    builder.stop()
