@@ -32,7 +32,8 @@ var firstDraw = true,
     animation_duration_s = null, // is calculated based on animation_duration_perc and tps.
     populate_god_agent_menu = null, // keep track of any new agents
     pop_new_chat_dropdown = null,
-    latest_tick_processed = null;
+    latest_tick_processed = null,
+    redraw_required = false; // for instance when the window has been resized
 
 // tracked HTML objects
 var saved_prev_objs = {}, // obj containing the IDs of objects and their visualization settings of the previous tick
@@ -40,6 +41,9 @@ var saved_prev_objs = {}, // obj containing the IDs of objects and their visuali
     bg_tile_ids = [], // obj_IDS of background tiles
     matrx_tile_ids = []; // obj_IDS of MATRX objects
 
+// track
+var object_selected = false; //
+
 /**
  * Get the grid wrapper object
  */
@@ -70,7 +74,7 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
     parse_world_settings(world_settings);
 
     // if we already processed this tick (MATRX is paused), stop and return
-    if (latest_tick_processed == current_tick) {
+    if (latest_tick_processed == current_tick && !redraw_required) {
         return;
     }
 
@@ -95,8 +99,8 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
         obj = state[objID];
 
         // get the location of the object in pixel values
-        var x = obj['location'][0] * tile_size;
-        var y = obj['location'][1] * tile_size;
+        var x = obj['location'][0];
+        var y = obj['location'][1];
 
         // fetch bg img if defined
         var obj_img = null;
@@ -104,6 +108,10 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
             obj_img = fix_img_url(obj['img_name']);
         }
 
+        var show_busy_condition =  (obj.hasOwnProperty("is_blocked_by_action") &&
+                                    obj['visualization'].hasOwnProperty('show_busy') &&
+                                    obj['visualization']['show_busy']);
+
         // save visualization settings for this object
         var obj_vis_settings = {
             "img": obj_img,
@@ -111,9 +119,17 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
             "size": obj['visualization']['size'], // percentage how much of tile is filled
             "colour": hexToRgba(obj['visualization']['colour'], obj['visualization']['opacity']),
             "opacity": obj['visualization']['opacity'],
-            "dimension": tile_size // width / height of the tile
+            "dimension": tile_size, // width / height of the tile
+            "busy": (show_busy_condition ? obj['is_blocked_by_action'] : false), // show busy if available and requested
+            "selected": (object_selected == objID ? true : false)
         };
 
+        // Check if any subtiles have been defined and include them in the ob_vis_settings if so
+        if (Object.keys(obj).includes('subtiles') && Object.keys(obj).includes('subtile_loc')) {
+            obj_vis_settings['subtiles'] = obj["subtiles"];
+            obj_vis_settings['subtile_loc'] = obj["subtile_loc"];
+        }
+
         var obj_element = null; // the html element of this object
         var animate_movement = false; // whether any x,y position changes should be animated
         var object_is_new = false; // whether this is a new object, not present in the html yet
@@ -136,9 +152,14 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
             grid.append(obj_element);
 
             // add this agent to the dropdown list
-            if (obj_element.hasOwnProperty('isAgent')) {
+            if (obj.hasOwnProperty('isAgent')) {
+
+                console.log("adding selection listener:", obj);
                 populate_god_agent_menu = true;
                 pop_new_chat_dropdown = true;
+
+                // add a click listener for the selection
+                add_selection_listener(obj_element);
             }
 
             // we already generated this object in a previous tick
@@ -148,7 +169,7 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
 
             // check if the coordinate changed compared to the previous tick, and if so, add css rules
             // for animating the x,y coordinates change
-            if (obj_element.style.left != (x * tile_size) || obj_element.style.top != (y * tile_size)) {
+            if (obj_element.style.left != (x * tile_size) + "px" || obj_element.style.top != (y * tile_size) + "px") {
                 obj_element.style.setProperty("-webkit-transition", "all " + animation_duration_s + "s");
                 obj_element.style.transition = "all " + animation_duration_s + "s";
 
@@ -160,22 +181,23 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
             // to (re)style the object
             if (compare_objects(saved_prev_objs[objID], obj_vis_settings)) {
                 style_object = false;
-
-                // repopulate the agent list, when the visualization settings changed of an agent
-                if (obj_element.hasOwnProperty('isAgent')) {
-                    populate_god_agent_menu = true;
-                    pop_new_chat_dropdown = true;
-                }
             }
+
         }
 
         // set the visualization depth of this object
         obj_element.style.zIndex = obj['visualization']['depth'];
-//        obj_element.style.zIndex = 0;
 
         // if we need to style this object, e.g. because it's new or visualiation settings changed,
         // regenerate the specfic object shape with its settings
-        if (style_object) {
+        if (style_object || redraw_required) {
+
+            // repopulate the agent list, when the visualization settings changed of an agent
+            if (obj.hasOwnProperty('isAgent')) {
+                populate_god_agent_menu = true;
+                pop_new_chat_dropdown = true;
+            }
+
             set_tile_dimensions(obj_element);
 
             // draw the object with the correct shape, size and colour
@@ -199,6 +221,9 @@ function draw(state, world_settings, new_messages, accessible_chatrooms, new_tic
         })
     });
 
+    // all objects have been redrawn, so this can be set to false again
+    redraw_required = false;
+
     // any objects present in the previous tick but not present in the current
     // tick should be removed
     saved_prev_obj_keys.forEach(function(objID) {
@@ -242,6 +267,9 @@ function fix_grid_size() {
         // resize the grid to exactly encompass all tiles
         grid.style.width = tile_size * grid_size[0] + "px";
         grid.style.height = tile_size * grid_size[1] + "px";
+
+        // next time redraw all objects
+        redraw_required = true;
     }
 
     console.log("Fixed tile and grid size");
@@ -335,7 +363,11 @@ function parse_world_settings(world_settings) {
  */
 function parse_vis_settings(vis_settings) {
     bg_colour = vis_settings['vis_bg_clr'];
-    bg_image = fix_img_url(vis_settings['vis_bg_img']);
+
+    bg_image = null;
+    if (Object.keys(vis_settings).includes('vis_bg_img') && vis_settings['vis_bg_img'] != null) {
+        bg_image = fix_img_url(vis_settings['vis_bg_img']);
+    }
 
     // update background colour / image if needed
     draw_bg();
@@ -507,13 +539,47 @@ function gen_rectangle(obj_vis_settings, obj_element, element_type = "div") {
     var shape = document.createElement(element_type);
     shape.className = "shape";
 
-    // coords of top left corner, such that it is centerd in our tile
-    shape.style.left = ((1 - size) * 0.5 * tile_size);
-    shape.style.top = ((1 - size) * 0.5 * tile_size);
+    // use subtiles if they are defined and their location is valid
+    if("subtiles" in obj_vis_settings
+            && "subtile_loc" in obj_vis_settings
+            && obj_vis_settings['subtile_loc'][0] >= 0
+            && obj_vis_settings['subtile_loc'][0] < obj_vis_settings['subtiles'][0]
+            && obj_vis_settings['subtile_loc'][1] >= 0
+            && obj_vis_settings['subtile_loc'][1] < obj_vis_settings['subtiles'][1]) {
 
-    // width and height of rectangle
-    shape.style.width = size * tile_size + "px";
-    shape.style.height = size * tile_size + "px";
+        // get width and height of subtile
+        tileW = tile_size * (1.0 / obj_vis_settings["subtiles"][0]);
+        tileH = tile_size * (1.0 / obj_vis_settings["subtiles"][1]);
+        var subtile_tile_sizes = [tileW, tileH];
+
+        // calc the offset of the subtile within the parent tile
+        var x_offset = tileW * (obj_vis_settings["subtile_loc"][0]);
+        var y_offset = tileH * (obj_vis_settings["subtile_loc"][1]);
+        var subtile_offsets = [x_offset, y_offset];
+
+        // offset of the topleft parent tile to the subtile + offset of the object within that subtile
+        shape.style.left = subtile_offsets[0] + ((1 - size) * 0.5 * subtile_tile_sizes[0]) + "px";
+        shape.style.top = subtile_offsets[1] + ((1 - size) * 0.5 * subtile_tile_sizes[1]) + "px";
+
+        // width and height of rectangle
+        shape.style.width = size * subtile_tile_sizes[0] + "px";
+        shape.style.height = size * subtile_tile_sizes[1] + "px";
+
+    // subtitle are defined but the subtile_loc is not valid, give a warning in the terminal
+    } else if ("subtiles" in obj_vis_settings && "subtile_loc" in obj_vis_settings) {
+        console.log("Object with subtiles", obj_vis_settings['subtiles'], ' has subtile_loc',
+                obj_vis_settings['subtile_loc'] , ' that is out of range. Drawing without subtile.');
+
+    // no subtiles defined, place the rectangle in the usual manner
+    } else {
+        // coords of top left corner, such that it is centered in our tile
+        shape.style.left = ((1 - size) * 0.5 * tile_size) + "px";
+        shape.style.top = ((1 - size) * 0.5 * tile_size) + "px";
+
+        // width and height of rectangle
+        shape.style.width = size * tile_size + "px";
+        shape.style.height = size * tile_size + "px";
+    }
 
     // styling
     shape.style.background = obj_vis_settings['colour'];
@@ -524,12 +590,35 @@ function gen_rectangle(obj_vis_settings, obj_element, element_type = "div") {
     }
 
     // add a click listener for the context menu
-    if (lv_agent_type != "agent") {
-        add_context_menu(shape);
-    }
+    add_context_menu(shape);
 
     // add the new shape
     obj_element.append(shape);
+
+    // check if we need to show a loading icon
+    if (obj_vis_settings['busy']) {
+        var busy_icon = document.createElement('img');
+        busy_icon.className = "matrx_object_busy";
+        // set the image source
+        busy_icon.setAttribute("src", '/static/images/busy_black_small.gif');
+        // small loading icon
+        busy_icon.style.width = 0.35 * tile_size + "px";
+        busy_icon.style.height = 0.35 * tile_size + "px";
+        // position in the top left corner
+        busy_icon.style.right = "0px";
+        busy_icon.style.top = "0px";
+        obj_element.append(busy_icon);
+    }
+
+    // add selection image if needed
+    if (obj_vis_settings['selected']) {
+        var selection_img = document.createElement('img');
+        selection_img.className = "matrx_object_selected";
+        // set the image source
+        selection_img.setAttribute("src", '/static/images/selected.png');
+        obj_element.append(selection_img);
+    }
+
     return shape;
 }
 
@@ -547,19 +636,57 @@ function gen_triangle(obj_vis_settings, obj_element) {
     var shape = document.createElement('div');
     shape.className = "shape";
 
-    // coords of top left corner, such that it is centerd in our tile
-    shape.style.left = ((1 - size) * 0.5 * tile_size);
-    shape.style.top = ((1 - size) * 0.5 * tile_size);
-
     // for a triangle, we set the content width/height to 0
     shape.style.width = 0;
     shape.style.height = 0;
 
-    // instead we use borders to create the triangle, it is basically the bottom border with
-    // the left and right side cut off diagonally by transparent borders on the side
-    shape.style.borderBottom = (size * tile_size) + "px solid " + obj_vis_settings['colour'];
-    shape.style.borderLeft = (size * tile_size / 2) + "px solid transparent";
-    shape.style.borderRight = (size * tile_size / 2) + "px solid transparent";
+    // use subtiles if they are defined and their location is valid
+    if("subtiles" in obj_vis_settings
+            && "subtile_loc" in obj_vis_settings
+            && obj_vis_settings['subtile_loc'][0] >= 0
+            && obj_vis_settings['subtile_loc'][0] < obj_vis_settings['subtiles'][0]
+            && obj_vis_settings['subtile_loc'][1] >= 0
+            && obj_vis_settings['subtile_loc'][1] < obj_vis_settings['subtiles'][1]) {
+
+        // get width and height of subtile
+        tileW = tile_size * (1.0 / obj_vis_settings["subtiles"][0]);
+        tileH = tile_size * (1.0 / obj_vis_settings["subtiles"][1]);
+        var subtile_tile_sizes = [tileW, tileH];
+
+        // calc the offset of the subtile within the parent tile
+        var x_offset = tileW * (obj_vis_settings["subtile_loc"][0]);
+        var y_offset = tileH * (obj_vis_settings["subtile_loc"][1]);
+        var subtile_offsets = [x_offset, y_offset];
+
+        // offset of the topleft parent tile to the subtile + offset of the object within that subtile
+        shape.style.left = subtile_offsets[0] + ((1 - size) * 0.5 * subtile_tile_sizes[0]) + "px";
+        shape.style.top = subtile_offsets[1] + ((1 - size) * 0.5 * subtile_tile_sizes[1]) + "px";
+
+        // We use borders to create the triangle, it is basically the bottom border with
+        // the left and right side cut off diagonally by transparent borders on the side
+        shape.style.borderBottom = (size * subtile_tile_sizes[0]) + "px solid " + obj_vis_settings['colour'];
+        shape.style.borderLeft = (size * subtile_tile_sizes[1] / 2) + "px solid transparent";
+        shape.style.borderRight = (size * subtile_tile_sizes[1] / 2) + "px solid transparent";
+
+
+    // subtitle are defined but the subtile_loc is not valid, give a warning in the terminal
+    } else if ("subtiles" in obj_vis_settings && "subtile_loc" in obj_vis_settings) {
+        console.log("Object with subtiles", obj_vis_settings['subtiles'], ' has subtile_loc',
+                obj_vis_settings['subtile_loc'] , ' that is out of range. Drawing without subtile.');
+
+    // no subtiles defined, place the triangle in the usual manner
+     } else {
+
+        // coords of top left corner, such that it is centered in our tile
+        shape.style.left = ((1 - size) * 0.5 * tile_size) + "px";
+        shape.style.top = ((1 - size) * 0.5 * tile_size) + "px";
+
+        // instead we use borders to create the triangle, it is basically the bottom border with
+        // the left and right side cut off diagonally by transparent borders on the side
+        shape.style.borderBottom = (size * tile_size) + "px solid " + obj_vis_settings['colour'];
+        shape.style.borderLeft = (size * tile_size / 2) + "px solid transparent";
+        shape.style.borderRight = (size * tile_size / 2) + "px solid transparent";
+    }
 
     // remove any old shapes
     while (obj_element.firstChild) {
@@ -567,12 +694,35 @@ function gen_triangle(obj_vis_settings, obj_element) {
     }
 
     // add a click listener for the context menu
-    if (lv_agent_type != "agent") {
-        add_context_menu(shape);
-    }
+    add_context_menu(shape);
 
     // add the new shape
     obj_element.append(shape);
+
+   // check if we need to show a loading icon
+    if (obj_vis_settings['busy']) {
+        var busy_icon = document.createElement('img');
+        busy_icon.className = "matrx_object_busy";
+        // set the image source
+        busy_icon.setAttribute("src", '/static/images/busy_black_small.gif');
+        // small loading icon
+        busy_icon.style.width = 0.35 * tile_size + "px";
+        busy_icon.style.height = 0.35 * tile_size + "px";
+        // position in the top left corner
+        busy_icon.style.right = "0px";
+        busy_icon.style.top = "0px";
+        obj_element.append(busy_icon);
+    }
+
+    // add selected img if needed
+    if (obj_vis_settings['selected']) {
+        var selection_img = document.createElement('img');
+        selection_img.className = "matrx_object_selected";
+        // set the image source
+        selection_img.setAttribute("src", '/static/images/selected.png');
+        obj_element.append(selection_img);
+    }
+
     return shape;
 }
 
@@ -610,22 +760,9 @@ function gen_image(obj_vis_settings, obj_element) {
     shape.style.background = "transparent";
     shape.style.opacity = obj_vis_settings["opacity"];
 
-    shape.dataset.toggle = "dropdown";
-
     return shape;
 }
 
-// add an event listener for mouse clicks, opening the context menu
-function add_context_menu(object) {
-    $(object).contextMenu({
-        menuSelector: "#contextMenu",
-        menuSelected: function(invokedOn, selectedMenu) {
-            var msg = "You selected the menu item '" + selectedMenu.text() +
-                "' on the value '" + invokedOn.text() + "'";
-            alert(msg);
-        }
-    });
-}
 
 
 /**
@@ -657,6 +794,9 @@ function draw_bg_tiles() {
             var pos_y = y * tile_size;
             tile.style = "left:" + pos_x + "px; top:" + pos_y + "px; width: " + tile_size + "px; height: " + tile_size + "px; z-index: 0;";
 
+            // add context menu
+            add_context_menu(tile);
+
             // add to grid
             grid.append(tile);
 
@@ -667,6 +807,94 @@ function draw_bg_tiles() {
 }
 
 
+/*********************************************************************
+ * Click listeners
+ ********************************************************************/
+
+// add an event listener for left mouse clicks, that selects an agent
+function add_selection_listener(object) {
+    $(object).click( function() {
+
+        console.log("clicked on selection listener");
+        var obj = $(this);
+        var obj_id = obj.attr('id');
+
+        // clicking twice on the same object deselects it
+        if (object_selected == obj_id) {
+            deselect(obj_id)
+
+        // deselect any other selected objects, and select the clicked object
+        } else {
+            if (object_selected != false) {
+                deselect(object_selected);
+            }
+
+            select(obj_id)
+        }
+    })
+
+}
+
+// select an object
+function select(obj_id) {
+    object_selected = obj_id;
+
+    // add selection image
+    var selection_img = document.createElement('img');
+    selection_img.className = "matrx_object_selected";
+    // set the image source
+    selection_img.setAttribute("src", '/static/images/selected.png');
+    $('#' + obj_id).append(selection_img);
+}
+
+// deselect an object
+function deselect(obj_id) {
+    object_selected = false;
+    document.getElementById(obj_id).style.backgroundImage = "none";
+
+    // remove selection image
+    $('#' + obj_id).find('.matrx_object_selected').remove()
+}
+
+// add an event listener for right mouse clicks, opening the context menu
+function add_context_menu(object) {
+    $(object).contextMenu({
+        menuSelector: "#contextMenu",
+        menuSelected: function(invokedOn, selectedMenu) {
+
+            // console.log("Execute API call with message:", selectedMenu[0].mssg)
+
+            var matrx_url = 'http://' + window.location.hostname,
+                port = "3001",
+                matrx_send_message_pickled = "send_message_pickled";
+
+            post_data = {'sender': lv_agent_id, 'message': selectedMenu[0].mssg}
+            // console.log("Sending post data:", post_data);
+
+            var context_menu_request = $.ajax({
+                method: "POST",
+                data: JSON.stringify(post_data),
+                url: matrx_url + ":" + port + "/" + matrx_send_message_pickled,
+                contentType: "application/json; charset=utf-8",
+                dataType: 'json'
+            });
+
+            // console.log("Response:", context_menu_request);
+
+            // if the request gave an error, print to console and try to reinitialize
+            context_menu_request.fail(function(data) {
+                console.log("Sending context menu input failed:", data.responseJSON)
+            });
+
+            // if the request was succesfull, add the options to the menu and show the menu
+            context_menu_request.done(function(data) {
+                // console.log("Sending context menu input succeeded:", data)
+            });
+
+        }
+    });
+}
+
 /*********************************************************************
  * Helper methods
  ********************************************************************/
@@ -717,8 +945,10 @@ function compare_objects(o1, o2) {
  * Move an object to a new x, y coordinate using css
  */
 function move_object(obj_element, x, y) {
-    obj_element.style.left = x + "px";
-    obj_element.style.top = y + "px";
+    obj_element.style.left = (x * tile_size) + "px";
+    obj_element.style.top = (y * tile_size) + "px";
+    obj_element.cell_x = x;
+    obj_element.cell_y = y;
 }
 
 /*
